// -*- c++ -*-
// $Id: loadointerp.C,v 1.3 2005/06/07 10:00:34 olsonse Exp $
/*
 * Copyright 2004-2005 Spencer Olson
 *
 * $Log: loadointerp.C,v $
 * Revision 1.3  2005/06/07 10:00:34  olsonse
 * Many changes over the course of time
 *
 * Revision 1.2  2005/01/19 12:04:46  olsonse
 * Added interrupt handler to stop loading.
 * Added options to specify start and index increment of data to save.
 *
 * Revision 1.1.1.1  2005/01/08 04:57:26  olsonse
 * Initial CVS import (finally).
 *
 *
 */


#include <dsmc/DSMCData.h>
#include <dsmc/OctreeInterpolator.h>
#include <fstream>
#include "interp.h"
#include <olson-tools/readData.h>
#include <signal.h> // so we can issue ^C to gently stop the loading
#include <octave/oct.h>
#include <octave/pager.h>

void interruptRead(int inSignal) {
    interrupt_read = true;
}

DEFUN_DLD (loadointerp, args, ,
"  iid = loadointerp(filename, NodalRadiusOfInfluence [, start-index [, interval]])\n"
"  Load Octree Interpolation Data.\n"
"\n"
"Copyright 2004 Spencer Olson:\n"
"\n"
"  Who knows what to believe with this code yet. :-)\n"
"\n"
"  filename is a string of the file to open\n"
"  NodalRadiusOfInfluence is the number of nodes to consider\n"
"                         when calculating radii of Influence\n"
"                         for each data node.\n"
"  start-index   :   Index of the first data block to return\n"
"  interval      :   How many data blocks to read in per returned.\n"
) {
    if (args.length() < 2 || args.length() > 4) {
        print_usage("loadointerp");
        return octave_value();
    }

    unsigned int start = 0,
                 interval = 1;

    if (args.length() > 2) {
        if (args(2).int_value() < 0) {
            octave_stdout << "invalid start index; must be > 0" << std::endl;
            print_usage("loadointerp");
            return octave_value();
        }
        start = (unsigned int)args(2).int_value();
    }

    if (args.length() > 3) {
        if (args(3).int_value() < 0) {
            octave_stdout << "invalid interval; must be > 0" << std::endl;
            print_usage("loadointerp");
            return octave_value();
        }
        interval = (unsigned int)args(3).int_value();
    }

    std::string filename (args(0).string_value ());
    int nodes_of_influence (args(1).int_value ());
    std::ifstream input (filename.c_str());

    /* prepare a backdoor. */
    sig_t old_signal_handler = signal(SIGINT, interruptRead);

    /* read stuff in */
    std::vector<DSMCData*> retval;
    readDataBlocks(input,retval,start,interval);

    /* remove backdoor. */
    (void)signal(SIGINT, old_signal_handler);

    input.close();
    octave_stdout << "read in " << retval.size() << " data blocks" << std::endl;

    if (retval.size()==0) {
        octave_stdout << "no data to load in '" << filename << '\'' << std::endl;
        print_usage("loadointerp");
        return octave_value();
    }

    /* now create interpolator for each block of data */
    interpolator_vector * interpolators = new interpolator_vector;

    try {
        for (unsigned int i = 0; i < retval.size(); i++) {
            octree::OctreeInterpolator<DSMCData,Particle> * interp =
                new octree::OctreeInterpolator<DSMCData,Particle>;
            interp->createDataOctree(retval[i]);
            interp->setRadiiOfInfluence(nodes_of_influence);
            interpolators->i.push_back(interp);
        }
    } catch ( std::exception & e ) {
        octave_stdout << "exception thrown while loading data\n"
                      << e.what() << std::endl;
    }

    double ointVAddress = (long)interpolators;

    return octave_value (ointVAddress);
}
