// -*- c++ -*-
// $Id: setrinfluence.C,v 1.2 2005/06/07 10:00:34 olsonse Exp $
/*
 * Copyright 2004-2005 Spencer Olson
 *
 * $Log: setrinfluence.C,v $
 * Revision 1.2  2005/06/07 10:00:34  olsonse
 * Many changes over the course of time
 *
 * Revision 1.1  2005/01/22 20:28:45  olsonse
 * Added octave method to change the radius of influence of a group of
 * OctreeInterpolator/s.
 *
 *
 */


#include <dsmc/DSMCData.h>
#include <dsmc/OctreeInterpolator.h>
#include "interp.h"
#include <octave/oct.h>
#include <octave/pager.h>
#include <fstream>

DEFUN_DLD (setrinfluence, args, ,
"  setrinfluence(iid, NodalRadiusOfInfluence)\n"
"  Evaluate Octree Interpolation.\n"
"\n"
"Copyright 2004 Spencer Olson:\n"
"\n"
"  Who knows what to believe with this code yet. :-)\n"
"\n"
"  iid : id of interpolators as returned by setrinfluence\n"
"  NodalRadiusOfInfluence is the number of nodes to consider\n"
) {
    if (args.length() != 2) {
        print_usage("setrinfluence");
        return octave_value();
    }

    /* now create interpolator for each block of data */
    interpolator_vector * interpolators = (interpolator_vector *)((long)(args(0).double_value ()));

    if (!interpolators || !interpolators->testValid()) {
        octave_stdout << "invalid iid" << std::endl;
        print_usage("setrinfluence");
        return octave_value();
    }

    int nodes_of_influence (args(1).int_value ());

    try {
        for (unsigned int indx = 0; indx < interpolators->i.size(); indx++) {
            interpolators->i[indx]->clearRadiiOfInfluence();
            interpolators->i[indx]->setRadiiOfInfluence(nodes_of_influence);
        }
    } catch ( std::exception & e ) {
        octave_stdout << "exception thrown while setting radius of influence\n"
                      << e.what() << std::endl;
    }

    return octave_value ();
}
