// -*- c++ -*-
// $Id: DSMCData.C,v 1.3 2005/06/07 10:00:06 olsonse Exp $
/*
 * Copyright 2004 Spencer Olson
 *
 * $Log: DSMCData.C,v $
 * Revision 1.3  2005/06/07 10:00:06  olsonse
 * Many changes over the course of time
 *
 * Revision 1.2  2005/01/19 04:31:21  olsonse
 * Added special trivial case of only one particle from which to obtain info such
 * as temperature.
 *
 * Revision 1.1.1.1  2005/01/08 04:57:25  olsonse
 * Initial CVS import (finally).
 *
 *
 */


#include "DSMCData.h"
#include <olson-tools/physical.h>
#include <olson-tools/readData.h>

template < class _Particle>
DSMCData<_Particle>::DSMCData() {
    zeroFields();
}

template < class _Particle>
DSMCData<_Particle>::DSMCData(const DSMCData & d) {
    memcpy(pos,d.pos, sizeof(pos));
    memcpy(&data, &d.data, sizeof (data));
    radiusOfInfluence = d.radiusOfInfluence;
    nearestNeighbors = d.nearestNeighbors;
    particle_type = d.particle_type;

    /* still set this to NULL */
    octreeNode = NULL;
    next = NULL;
}

template < class _Particle>
DSMCData<_Particle>::DSMCData(const _Particle & q) {
    memcpy(pos,q.pos, sizeof(pos));
    zeroFields();
}

/* TODO:  Pick some good value for the mem-limit */
REDEF_NEW_DEL_C_TEMPLATE(template<class _Particle>,DSMCData<_Particle>,/* freed mem limit (instances) = */ 100000);

template < class _Particle>
std::ostream & operator<<(std::ostream & output,const DSMCData<_Particle> & d){
    output
    	<< d.pos[X] << '\t' << d.pos[Y] << '\t' << d.pos[Z];
    for (int i = 0; i < DSMCData<_Particle>::END_DATA; i++) {
    	output << '\t' << d.data.indx[i];
    }
    output << '\t' << d.radiusOfInfluence << '\t' << d.nearestNeighbors;
    return output;
} // operator<< Particle

/** Overloaded stream input for DSMCData\s for use with readData and
 * readDataBlocks.
 * @see readData and readDataBlocks.
 * */
template < class _Particle>
std::istream & operator>>(std::istream & input, DSMCData<_Particle> * & d) {
    while (read_past_comment(input));

    if (!input.good()) {
        /* for some reason, my new gcc (3.3.4) on SuSe 9.2 adds a bad element
         * on the end of the array from copy(...).  We need to be able to
         * therefore detect a bad element.  So let's make sure that we return
         * a NULL. */
        d = NULL;
        return input;
    }

    d = new DSMCData<_Particle>;
    return input >> d->pos[X]
                 >> d->pos[Y]
                 >> d->pos[Z]
                 >> d->data.name.numberDensity
                 >> d->data.name.N
                 >> d->data.name.max_sigma_relspeed
                 >> d->data.name.number_collision_pairs_remainder
                 >> d->data.name.collisions
                 >> d->data.name.avgv[X]
                 >> d->data.name.avgv[Y]
                 >> d->data.name.avgv[Z]
                 >> d->data.name.avgv2[X]
                 >> d->data.name.avgv2[Y]
                 >> d->data.name.avgv2[Z]
                 >> d->radiusOfInfluence
                 >> d->nearestNeighbors;
}

