// -*- c++ -*-
// $Id: ParticleSrc.C,v 1.3 2005/06/07 10:00:15 olsonse Exp $
/*
 * Copyright 2002-2004 Spencer Olson
 *
 * This is taken from the older atomguide code.
 *
 * $Log: ParticleSrc.C,v $
 * Revision 1.3  2005/06/07 10:00:15  olsonse
 * Many changes over the course of time
 *
 * Revision 1.2  2005/01/19 11:45:43  olsonse
 * Small fixes.
 *
 * Revision 1.1.1.1  2005/01/08 04:57:25  olsonse
 * Initial CVS import (finally).
 *
 *
 */

#include "ParticleSrc.h"
#include <math.h>

#define DFLT_PULSE_DURATION	1*physical::unit::ms

template <class SpatialDistribution, class VDistros, class _Particle>
void ParticleSrc<SpatialDistribution,VDistros,_Particle>::set_defaults() {
    flux = 1e9;	/* default to 10^9 atoms/s */
    pulsed_source = false;
    pulse_duration = DFLT_PULSE_DURATION;
    pulse_timer = 0.0;
    seed = _Particle();
    normalized_flux = 0.0;
    running_total_flux = 0.0;
    total_time = 0.0;

    origin = 0.0;       /* default to 0,0,0 */
    temperature = (150.0 * physical::unit::uK);	/* default to 150 uK */
    velocity = 0.0;      /* default to 0 m/s */

    v_range.min = -4.0;
    v_range.max =  4.0;

    nbins = 10000;
}

template <class SpatialDistribution, class VDistros, class _Particle>
_Particle * ParticleSrc<SpatialDistribution,VDistros,_Particle>::createParticles(const double & dt) {
    /* spawn the correct number of random-distributed atoms for a
     * time dt with a given flux. */
    long long flux_dt = (long long) (normalized_flux * dt + 0.5); /* normalized to a macroatom */
    long long n = 0;
    _Particle * retval = NULL;

    if(pulsed_source) {
	if (pulse_timer < pulse_duration) {
	    pulse_timer += dt;
	    if (pulse_timer > pulse_duration) {
		/* recalculate the flux for the last of the pulse */
		flux_dt = (long long) (normalized_flux * (dt - (pulse_timer-pulse_duration)) + 0.5);
	    }
	} else {
	    /* return because the pulse is finished. */
	    return NULL;
	}
    }

    while(n < flux_dt) {
	_Particle * p = new _Particle(seed);

	p->vel[X] = vdistro[X]->lever();
	p->vel[Y] = vdistro[Y]->lever();
	p->vel[Z] = vdistro[Z]->lever();

        p->pos[X] = pos_distrib.x() + origin[X];
        p->pos[Y] = pos_distrib.y() + origin[Y];
        p->pos[Z] = pos_distrib.z() + origin[Z];

	/* now compute the running flux total.
	 * Note that the flux is also normalized to the macroatom.
	 * */
	n++;

	p->next = retval;
	retval = p;
    }

    total_time += dt;
    running_total_flux += n * seed.size();

    /* we had no need to split up the tasks of this module */
    return retval;
}

template <class SpatialDistribution, class VDistros, class _Particle>
void ParticleSrc<SpatialDistribution,VDistros,_Particle>::reportStatistics(std::ostream & output) const {
    output << "Total Average Flux:                    " <<
	    (running_total_flux / total_time)
	   << std::endl;
}

